import os
import boto3
from urllib.parse import urlparse
from paperqa import Docs, Text, Doc  
from langchain.chat_models import ChatAnthropic, ChatOpenAI

def knowledge_base(documents, temperature=0.7, is_text=False):
    # Initialize the language models
    gpt4 = ChatOpenAI(model='gpt-4', temperature=temperature)
    claude = ChatAnthropic(model="claude-2", temperature=temperature)
    llm = Docs(llm=gpt4, summary_llm=claude)

    if is_text:
        # Create a Doc object
        doc = Doc(docname="MyTextDoc", citation="My Citation", dockey="MyDocKey")
        text = Text(text=documents, name="MyTextName", doc=doc)
        llm.add_texts([text], doc)
        return llm
    
    s3 = boto3.client('s3')

    for path in documents:
        parsed_url = urlparse(path)
        scheme = parsed_url.scheme
        ext = os.path.splitext(path)[-1].lower()

        if scheme == 's3':
            bucket = parsed_url.netloc
            key = parsed_url.path.lstrip('/')

            if key.endswith('/'):  # It's a directory
                # List all objects in the directory
                objects = s3.list_objects(Bucket=bucket, Prefix=key)
                for obj in objects.get('Contents', []):
                    file_key = obj['Key']
                    local_filename = os.path.basename(file_key)
                    if local_filename:  # Skip if it's not a valid filename
                        local_path = f"/tmp/{local_filename}"
                        s3.download_file(bucket, file_key, local_path)
                        llm.add(local_path)
            else:  # It's a file
                local_path = f"/tmp/{os.path.basename(key)}"
                s3.download_file(bucket, key, local_path)
                llm.add(local_path)

        elif scheme in ['http', 'https']:
            llm.add_url(path)
        else:
            if ext == '.pdf':
                llm.add(path)
            else:
                with open(path, 'r') as f:
                    text_content = f.read()
                    llm.add_file(f, text_content)

    return llm


"""autopop: Auto Populate GDoc tables using LLMs."""

__version__ = '2.5'

from .aspect_factory import AspectFactory
from .aspect_factory import dynamic_init, get_instances
from .ddb_util_wrapper import DDBUtilWrapper
from .google_util_wrapper import GoogleUtilWrapper
from .google_util_wrapper import call_post_sheet, get_bsme_interview
from .google_util_wrapper import get_tsme_interview, get_related_docs
from .knowledge_base import knowledge_base

__all__ = [
    'AspectFactory',
    'GoogleUtilWrapper',
    'DDBUtilWrapper',
    'knowledge_base',
    'call_post_sheet',
    'get_bsme_interview',
    'get_tsme_interview',
    'get_related_docs',
    'dynamic_init', 
    'get_instances'
]


import time
import json
import boto3
import requests
from urllib.parse import urlparse

def call_post_sheet(instance, sheet=None, column=None, use_prompt=False): 
    instance.post_sheet(sheet=sheet, column=column, use_prompt=use_prompt)    

def get_bsme_interview(self):
    return gu.get_data('bsme')

def get_tsme_interview(self):
    return gu.get_data('tsme')

def get_related_docs(gu, s3_path):
    s3 = boto3.client('s3')
    parsed_url = urlparse(s3_path)
    s3_bucket = parsed_url.netloc
    s3_directory = parsed_url.path.lstrip('/')
    filenames = []

    for k, v in gu.get_data('related').items():
        data = json.dumps(gu.get_data(p2_url=v))
        s3_key = f"{s3_directory}{k}.json" 
        s3.put_object(Body=data, Bucket=s3_bucket, Key=s3_key)
        filenames.append(s3_key)
    return filenames

def get_related_docs(gu, s3_path):
    s3 = boto3.client('s3')
    parsed_url = urlparse(s3_path)
    s3_bucket = parsed_url.netloc
    s3_directory = parsed_url.path.lstrip('/')
        
    for k, v in gu.get_data('related').items():
        data = gu.get_data(p2_url=p2 + url)        
        s3_key = f"{s3_directory}/{k}.json"  # Assuming the data is JSON
        s3.put_object(Body=data_str, Bucket=s3_bucket, Key=s3_key)

class GoogleUtilWrapper:
    def __init__(self, p2_url=None, gdt_url=None, pdest=None, gdest=None):
        self.p2_url = p2_url
        self.gdt_url = gdt_url
        self.pdest = pdest
        self.gdest = gdest

    def get_data(self, ep='', fields=None, p2_url=None):
        payload = {}
        payload["p2"] = self.p2_url if not p2_url else p2_url
        if fields: payload["fields"] = fields
        response = requests.get(self.gdt_url + ep, json=payload)
        return response.json()

    def post_doc_data(self, fields=[], data=None):
        payload = {}
        payload["p2"] = self.p2_url
        payload["fields"] = fields
        payload["destination"] = self.pdest
        if data: payload["data"] = data
        response = requests.post(self.gdt_url, json=payload)

    def post_sheet_data(self, fields=[], aspects=[], sheet=None, column=None, data=None):
        if aspects:
            fields = [{"table": aspect['table'], "row": row} for aspect in aspects for row in aspect['row']]
        payload = {}
        payload["p2"] = self.p2_url
        payload["fields"] = fields
        payload["destination"] = self.gdest
        if data: payload["data"] = data
        if sheet: payload["sheet"] = sheet
        if column: payload["source"] = column
        response = requests.post(self.gdt_url, json=payload)